"""This Python module implements the code related to the following scientific paper:

F. Ragusa, A. Furnari, S. Battiato, G. Signorello, G. M. Farinella, Egocentric Visitors Localization in Cultural Sites, submitted to ACM Journal on Computing and Cultural Heritage (JOCCH)

More information is available at our web page: http://iplab.dmi.unict.it/VEDI/

The module is designed to work with Python 2.7"""


# set up Python environment: numpy for numerical routines, and matplotlib for plotting
import numpy as np


# The caffe module needs to be on the Python path;
#  we'll add it here explicitly.
import sys
caffe_root = '/home/fragusa/caffe/'  # this file should be run from {caffe_root}/examples (otherwise change this line)
sys.path.insert(0, caffe_root + 'python')

import caffe
import os
import glob
import shutil

caffe.set_mode_gpu()
print 'GPU Mode'

dir = "1_Discrimination/output"
if os.path.exists(dir):
	shutil.rmtree(dir)
os.makedirs(dir)


model_def = '1_Discrimination/GoPro/VGG_19_VEDI_deploy.prototxt'
model_weights = '1_Discrimination/GoPro/_iter_100000.caffemodel'


n_images = len(os.listdir("frames"))
n_class = 9
posterior_probs = np.zeros( (n_images,n_class) )
out = open("1_Discrimination/output/filename.txt", "a")
out1 = open("1_Discrimination/output/CNN_classification.txt", "a")

net = caffe.Net(model_def,      # defines the structure of the model
                model_weights,  # contains the trained weights
                caffe.TEST)     # use test mode (e.g., don't perform dropout)

# load the mean ImageNet image (as distributed with Caffe) for subtraction
mu = np.load(caffe_root + 'python/caffe/imagenet/ilsvrc_2012_mean.npy')
mu = mu.mean(1).mean(1)  # average over pixels to obtain the mean (BGR) pixel values
print 'mean-subtracted values:', zip('BGR', mu)

# create transformer for the input called 'data'
transformer = caffe.io.Transformer({'data': net.blobs['data'].data.shape})

transformer.set_transpose('data', (2,0,1))  # move image channels to outermost dimension
transformer.set_mean('data', mu)            # subtract the dataset-mean value in each channel
transformer.set_raw_scale('data', 255)      # rescale from [0, 1] to [0, 255]
transformer.set_channel_swap('data', (2,1,0))  # swap channels from RGB to BGR

# set the size of the input (we can skip this if we're happy
#  with the default; we can also change it later, e.g., for different batch sizes)
net.blobs['data'].reshape(50,        # batch size
                          3,         # 3-channel (BGR) images
                          224, 224)  # image size is 227x227


list_images = sorted(glob.glob('frames/*.jpg'))
for i in range(0,len(list_images)):

	out.write(list_images[i]+'\n')
print 'fatto'
for i in range(0,n_images):

	image = caffe.io.load_image(list_images[i])
	transformed_image = transformer.preprocess('data', image)


	# copy the image data into the memory allocated for the net
	net.blobs['data'].data[...] = transformed_image

	### perform classification
	output = net.forward()

	output_prob = output['prob'][0]  # the output probability vector for the first image in the batch
	posterior_probs[i] = output_prob
	print 'predicted class is:', output_prob
	
	#out.write((str)output_prob)
	# load ImageNet labels
	labels_file = 'classes.txt'
	labels = np.loadtxt(labels_file, str, delimiter='\t')
	print 'output label:', labels[output_prob.argmax()]
	print >>out1, 'output label:', labels[output_prob.argmax()] 
	print str(i)+'/'+str(n_images)
	


np.savetxt('1_Discrimination/output/posterior_probs.txt',posterior_probs)
# sort top five predictions from softmax output
#top_inds = output_prob.argsort()[::-1][:5]  # reverse sort and take five largest items

#print 'probabilities and labels:'
#zip(output_prob[top_inds], labels[top_inds])

out.close()
out1.close()



#FILTER CNN_classification.txt in CNN_classification_mod.txt

classification = []
classification_mod = []
with open('1_Discrimination/output/CNN_classification.txt', "r") as inFile:
	for line in inFile:
		classification.append(line.split()) 


for i in range(0,len(classification)):
	classification_mod.append(classification[i][2])

out= open("1_Discrimination/output/CNN_classification_mod.txt", "a")

for i in range(0,len(classification_mod)):
	out.write(classification_mod[i]+'\n')
		
out.close()

#####

